/*******************************************************************************
 * HY16F198B
 * -----------------------------------------------------------------------------
 * HYCON Technology
 * http://www.hycontek.com/
 * Author : FAE/Robert Wang
 * C Library release V1.0
 * ClucoseMeter Demo Code V0.1
 * Maintenance:2016/06/21
 ******************************************************************************/

/*----------------------------------------------------------------------------*/
/* Includes                                                                   */
/*----------------------------------------------------------------------------*/
#include "HY16F198.h"
#include "DrvADC.h"
#include "DrvClock.h"
#include "DrvGPIO.h"
#include "DrvLCD.h"
#include "DrvPMU.h"
#include "ModuleID.h"
#include "Sysinfra.h"
#include "System.h"
#include "Display.h"
#include "my define.h"
#include "DrvREG32.h"
#include "DrvOP.h"
#include "DrvDAC.h"
#include "DrvFlash.h"
/*----------------------------------------------------------------------------*/
/* STRUCTURES                                                                 */
/*----------------------------------------------------------------------------*/
volatile typedef union _MCUSTATUS
{
  char  _byte;
  struct
  {
    unsigned b_ADCdone:1;
    unsigned b_TMAdone:1;
    unsigned b_TMBdone:1;
    unsigned b_TMC0done:1;
    unsigned b_TMC1done:1;
    unsigned b_RTCdone:1;
    unsigned b_UART_TxDone:1;
    unsigned b_UART_RxDone:1;
  };
} MCUSTATUS;

typedef union _PTINTSTATUS
{
  char  _byte;
  struct
  {
    unsigned b_PTINT0done:1;
    unsigned b_PTINT1done:1;
    unsigned b_PTINT2done:1;
    unsigned b_PTINT3done:1;
    unsigned b_PTINT4done:1;
    unsigned b_PTINT5done:1;
    unsigned b_PTINT6Done:1;
    unsigned b_PTINT7Done:1;
  };
} PTINTSTATUS;

/*----------------------------------------------------------------------------*/
/* DEFINITIONS                                                                */
/*----------------------------------------------------------------------------*/
//#define ADC_Count_show_19bit //to show ADC_Count
#define ADC_ohm_show_19bit  //to show measure ohm
#define DAC_Calibration_teaching_mode
//#define DAC_Calibration_fast_mode

/*----------------------------------------------------------------------------*/
/* Global CONSTANTS                                                           */
/*----------------------------------------------------------------------------*/
MCUSTATUS  MCUSTATUSbits;
PTINTSTATUS  PT1INTSTATUSbits;

unsigned char i;
unsigned char PT1_7_high;
unsigned char PT1_7_low;
unsigned char DAC_Output;
unsigned char ADfinish;
unsigned char DAC_Calibration_Flag;

int ADC_Array[2];
int ADCData_Chopper;
int DAC_ADCData;
int DAC_Calibration_ADCData;
int ADCData;
int K_ohm_ADCount[4];

float K_ohm[4];
float ADCData_temp;
float ADCData_ohm_show;
float ADC_over_19bit;


/*----------------------------------------------------------------------------*/
/* Function PROTOTYPES                                                        */
/*----------------------------------------------------------------------------*/
void InitalADC(void);  //ADC initial for Measure RS_Strip
void InitalADC_DAC_Calibration(void);  //ADC initial for auto DAC Calibration
void Delay(unsigned int num);

/*----------------------------------------------------------------------------*/
/* Note Something                                                             */
/*----------------------------------------------------------------------------*/
//此Code是在RStrip=3k效正, 3k ohm 電組應該是要非常的準
//因為目前此線路分壓設計, DAC最大輸入電壓是0.38V, 當DAC輸出255, ADC Count最大量測是160500左右


/*----------------------------------------------------------------------------*/
/* MAIN  function                                                             */
/*----------------------------------------------------------------------------*/
int main(void)
{

  //ADC chopper temp data
  for(i=0;i<=1;i++)
  {
	ADC_Array[i]=0;
	ADC_Array[i]=0;
  }

  //S1=PT1.1 setting. If press S1 that means to save the DAC calibration voltage.
  DrvGPIO_ClearIntFlag(E_PT1,0x02);       //clear PT1.1 interrupt flag
  DrvGPIO_Open(E_PT1,0x02,E_IO_INPUT);    //set PT1.1 INPUT
  DrvGPIO_Open(E_PT1,0x02,E_IO_PullHigh); //enable PT1.1 pull high
  DrvGPIO_Open(E_PT1,0x02,E_IO_IntEnable);  //PT1.1 interrupt enable
  DrvGPIO_ClkGenerator(E_HS_CK,1);         //Set IO sampling clock input source is HS_CK
  DrvGPIO_IntTrigger(E_PT1,0x02,E_N_Edge);  //PT1.1 interrupt trigger method is negative edge

  //PT1.7 setting. If PT1.7=Low, To do DAC Auto-Calibration
  DrvGPIO_Open(E_PT1,0x80,E_IO_INPUT);    //set PT1.7 INPUT
  DrvGPIO_Open(E_PT1,0x80,E_IO_PullHigh); //enable PT1.7 pull high
  PT1_7_high=0x82; //0x82 : PT1.7=High, PT1.1=High
  PT1_7_low=0x02; //0x02 : PT1.7=Low, PT1.1=High

  //check DAC calibration time
  //DrvGPIO_Open(E_PT2,0x08,E_IO_OUTPUT);  //PT2.3 Output

//if PT1.7=Low
if(pio1_3 == PT1_7_low) //if PT1.7=Low. To to auto DAC Calibration by ADC measure
{
  //Default DAC Output Setting
  DAC_Output=0; //default DAC = 0/255
  DAC_ADCData=0;
  DAC_Calibration_ADCData=0;
  DAC_Calibration_Flag=0;

  DrvDAC_Open(E_DAC_PAIO4,E_DAC_NAIO7,DAC_Output);    //DAC_Vrefp= VDDA, DAC_Vrefn= VSSA, IF DAO=0
  DrvDAC_SetoutputIO(ENABLE);       //DAC output with PT3.1
  DrvDAC_EnableOutput();            //DAC output enable
  DrvDAC_Enable();                  //DAC IP enable

  // R2ROPA Setting
  DrvOP_PInput(E_OPP_DAO);
  DrvOP_NInput(E_OPN_AIO5);
  DrvOP_Open();
  DrvOP_OPOoutEnable(); //OPA OPEO=1

  InitalADC_DAC_Calibration();
  DisplayInit();
  ClearLCDframe();

}
//if PT1.7=high
else  //if PT1.7!=Low  //ADC Measure RS_Strip
{
  ADC_over_19bit=262143; //The max ADC Count +262143
  ADfinish=0;
  DAC_Output=228;  //default DAC = 228/255
  DAC_Calibration_Flag=0;

  //DAC Output Setting
  DrvDAC_Open(E_DAC_PAIO4,E_DAC_NAIO7,DAC_Output);    //DAC_Vrefp= VDDA, DAC_Vrefn= VSSA, IF DAO=0
  DrvDAC_SetoutputIO(ENABLE);       //DAC output with PT3.1
  DrvDAC_EnableOutput();            //DAC output enable
  DrvDAC_Enable();                  //DAC IP enable

  // R2ROPA Setting
  DrvOP_PInput(E_OPP_DAO);
  DrvOP_NInput(E_OPN_AIO5);
  DrvOP_Open();
  DrvOP_OPOoutEnable(); //OPA OPEO=1
}

  SYS_EnableGIE(7,0x1FF);  //Enable GIE(Global Interrupt)
  MCUSTATUSbits._byte = 0;
  PT1INTSTATUSbits._byte = 0;

  while((pio1_3<=PT1_7_low)&&(DAC_Calibration_Flag==0)) //0x02, if PT1.7=Low, PT1.1=High
  {

	if(MCUSTATUSbits.b_ADCdone)
	{
		while(!ADfinish);
		DAC_ADCData=ADCData;
	    LCD_DATA_DISPLAY(DAC_ADCData);	//To Do DAC Calibration
		ADfinish=0;
		MCUSTATUSbits.b_ADCdone=0;
	}

	  if(PT1INTSTATUSbits.b_PTINT0done)    //0x00, if PT1.7=low, PT1.1=low
	  {

#if defined(DAC_Calibration_teaching_mode)
	   	PT1INTSTATUSbits.b_PTINT0done=0;
	   	DAC_Calibration_ADCData=DAC_ADCData; //if goal is 330mV, DAC_ADCData=0x24fdf
	    LCD_DATA_DISPLAY(0);
	    Delay(100000);
	    LCD_DATA_DISPLAY(1);
	    Delay(100000);
	    LCD_DATA_DISPLAY(2);
	    Delay(100000);
	    LCD_DATA_DISPLAY(3);
	    Delay(100000);
	    LCD_DATA_DISPLAY(4);
	   	Delay(100000);
	    LCD_DATA_DISPLAY(5);
	   	Delay(100000);
	    LCD_DATA_DISPLAY(6);
	   	Delay(100000);
	   	LCD_DATA_DISPLAY(7);
	    Delay(100000);
	   	LCD_DATA_DISPLAY(8);
	   	Delay(100000);
	   	LCD_DATA_DISPLAY(9);
	   	Delay(100000);
	   	LCD_DATA_DISPLAY(10);
	   	Delay(100000);

	    int Vdac_OutputMin=0x00;
	  	int Vdac_OutputMax=0xff;
	  	int Vdac_Initial=0;
	  	int Vdac_Buffer=0;
	  	int Vdac_Output=0;

	   	Vdac_Initial=(Vdac_OutputMax+Vdac_OutputMin)/2;  //Vdac_Initial=127
	   	DrvDAC_Open(E_DAC_PAIO4,E_DAC_NAIO7,Vdac_Initial);    //DAC_Vrefp= VDDA, DAC_Vrefn= VSSA
	   	Vdac_Buffer=dac_04+1;	//Vdac_Buffer=127+1=128

	   	ADfinish=0;
	   	for(i=1;i<=8;i++)
	   	{
	   		while(!ADfinish);  //if ADfinish=0, stop at here

	   	    if(i<=7 && DAC_Calibration_ADCData<= ADCData)	//if Vin<Vdac,  Vdac Output code - Vdac_Buffer
	   	    {
              DrvADC_Disable();
              DrvADC_CombFilter(DISABLE);  //Disable CombFilter
	   	      Vdac_Buffer=Vdac_Buffer/2;
	   		  Vdac_Output=dac_04;
	   		  Vdac_Output=Vdac_Output-Vdac_Buffer;
	   		  dac_04=0xff00+Vdac_Output; //To do Vdac output voltage
	   		  DrvADC_Enable();
	   		  DrvADC_CombFilter(ENABLE);  //Disable CombFilter
	   		  DrvADC_ClearIntFlag();
	   		  DrvADC_EnableInt();
	   		  while(!ADfinish);
	   	    }

	   		else if(i<=7 && DAC_Calibration_ADCData>= ADCData)  //if Vin>Vdac, Vdac Output code + Vdac_Buffer
	   	    {
	   		  DrvADC_Disable();
	   		  DrvADC_CombFilter(DISABLE);  //Disable CombFilter
	   		  Vdac_Buffer=Vdac_Buffer/2;
	   		  Vdac_Output=dac_04;
	   		  Vdac_Output=Vdac_Output+Vdac_Buffer;
	   		  dac_04=0xff00+Vdac_Output;  //To do Vdac output voltage
	   		  DrvADC_Enable();
	   		  DrvADC_CombFilter(ENABLE);  //Disable CombFilter
	   		  DrvADC_ClearIntFlag();
	   		  DrvADC_EnableInt();
	   		  while(!ADfinish);
	   	    }

	   		if(i==8 && DAC_Calibration_ADCData<= ADCData) //if Vin<Vdac, Vdac Output code + 0
	   	    {
	   		  DrvADC_Disable();
	   		  DrvADC_CombFilter(DISABLE);  //Disable CombFilter
	   		  Vdac_Buffer=0;
	   		  Vdac_Output=dac_04;
	   		  Vdac_Output=Vdac_Output+Vdac_Buffer;
	   		  dac_04=0xff00+Vdac_Output;  //To do Vdac output voltage
	   		  DrvFlash_Burn_Word(0xf000,2000,Vdac_Output);  //DAC calibration Burn_Word at 0x9f000
	   		  DAC_Calibration_Flag=1; //If DAC_Calibration_Flag=1, leave while((pio1_3<=PT1_7_low)&&(DAC_Calibration_Flag==0))
	   		  DrvADC_Enable();
	   		  DrvADC_CombFilter(ENABLE);  //Disable CombFilter
	   		  DrvADC_ClearIntFlag();
	   		  DrvADC_EnableInt();
	   		  while(!ADfinish);
	   	    }

	   		if(i==8 && DAC_Calibration_ADCData>= ADCData)  //if Vin>Vdac, Vdac Output code + 1
	   	    {
	   		  DrvADC_Disable();
	   		  DrvADC_CombFilter(DISABLE);  //Disable CombFilter
	   		  Vdac_Buffer=1;
	   		  Vdac_Output=dac_04;
	   		  Vdac_Output=Vdac_Output+Vdac_Buffer;
	   		  dac_04=0xff00+Vdac_Output;  //To do Vdac output voltage
	   		  DrvFlash_Burn_Word(0xf000,2000,Vdac_Output);  //DAC calibration Burn_Word at 0x9f000
	   		  DAC_Calibration_Flag=1; //If DAC_Calibration_Flag=1, leave while((pio1_3<=PT1_7_low)&&(DAC_Calibration_Flag==0))
	   		  DrvADC_Enable();
	   		  DrvADC_CombFilter(ENABLE);  //Disable CombFilter
	   		  DrvADC_ClearIntFlag();
	   		  DrvADC_EnableInt();
	   		  while(!ADfinish);
	   	    }

	   	    LCD_DATA_DISPLAY(ADCData);	// Doing DAC Calibration, and show ADCData
	   		ADfinish=0;
	   		MCUSTATUSbits.b_ADCdone=0;
	   	}
#endif

#if defined(DAC_Calibration_fast_mode)
	   	PT1INTSTATUSbits.b_PTINT0done=0;
	   	DAC_Calibration_ADCData=DAC_ADCData; //if goal is 330mV, Reference DAC_ADCData=0x24fdf
        // check DAC calibration time
        //DrvGPIO_SetPortBits(E_PT2,0x08);  //PT2.3=High
        //Delay(1000); //wait ? second
        //DrvGPIO_ClrPortBits(E_PT2,0x08);  //PT2.3=Low
	   	LCD_DATA_DISPLAY(0);
	    Delay(100000); //Delay(100000)=wait 500ms
        // check DAC calibration time
        //DrvGPIO_SetPortBits(E_PT2,0x08);  //PT2.3=High
        //Delay(1000); //wait ? second
        //DrvGPIO_ClrPortBits(E_PT2,0x08);  //PT2.3=Low
	    int Vdac_OutputMin=0x00;
	  	int Vdac_OutputMax=0xff;
	  	int Vdac_Initial=0;
	  	int Vdac_Buffer=0;
	  	int Vdac_Output=0;

	   	Vdac_Initial=(Vdac_OutputMax+Vdac_OutputMin)/2;  //Vdac_Initial=127
	   	DrvDAC_Open(E_DAC_PAIO4,E_DAC_NAIO7,Vdac_Initial);    //DAC_Vrefp= VDDA, DAC_Vrefn= VSSA
	   	Vdac_Buffer=dac_04+1;	//Vdac_Buffer=127+1=128

	   	ADfinish=0;
	   	for(i=1;i<=8;i++)
	   	{
	   		while(!ADfinish);  //if ADfinish=0, stop at here

	   	    if(i<=7 && DAC_Calibration_ADCData<= ADCData)	//if Vin<Vdac,  Vdac Output code - Vdac_Buffer
	   	    {
              DrvADC_Disable();
              DrvADC_CombFilter(DISABLE);  //Disable CombFilter
	   	      Vdac_Buffer=Vdac_Buffer/2;
	   		  Vdac_Output=dac_04;
	   		  Vdac_Output=Vdac_Output-Vdac_Buffer;
	   		  dac_04=0xff00+Vdac_Output; //To do Vdac output voltage
	   		  DrvADC_Enable();
	   		  DrvADC_CombFilter(ENABLE);  //Disable CombFilter
	   		  DrvADC_ClearIntFlag();
	   		  DrvADC_EnableInt();
	   		  while(!ADfinish);
	   	    }

	   		else if(i<=7 && DAC_Calibration_ADCData>= ADCData)  //if Vin>Vdac, Vdac Output code + Vdac_Buffer
	   	    {
	   		  DrvADC_Disable();
	   		  DrvADC_CombFilter(DISABLE);  //Disable CombFilter
	   		  Vdac_Buffer=Vdac_Buffer/2;
	   		  Vdac_Output=dac_04;
	   		  Vdac_Output=Vdac_Output+Vdac_Buffer;
	   		  dac_04=0xff00+Vdac_Output;  //To do Vdac output voltage
	   		  DrvADC_Enable();
	   		  DrvADC_CombFilter(ENABLE);  //Disable CombFilter
	   		  DrvADC_ClearIntFlag();
	   		  DrvADC_EnableInt();
	   		  while(!ADfinish);
	   	    }

	   		if(i==8 && DAC_Calibration_ADCData<= ADCData) //if Vin<Vdac, Vdac Output code + 0
	   	    {
	   		  DrvADC_Disable();
	   		  DrvADC_CombFilter(DISABLE);  //Disable CombFilter
	   		  Vdac_Buffer=0;
	   		  Vdac_Output=dac_04;
	   		  Vdac_Output=Vdac_Output+Vdac_Buffer;
	   		  dac_04=0xff00+Vdac_Output;  //To do Vdac output voltage
	   		  DrvFlash_Burn_Word(0xf000,2000,Vdac_Output);  //DAC calibration Burn_Word at 0x9f000
	   		  DAC_Calibration_Flag=1; //If DAC_Calibration_Flag=1, leave while((pio1_3<=PT1_7_low)&&(DAC_Calibration_Flag==0))
              // check DAC calibration time
              //DrvGPIO_SetPortBits(E_PT2,0x08);  //PT2.3=High
              //Delay(1000); //wait ? second
              //DrvGPIO_ClrPortBits(E_PT2,0x08);  //PT2.3=Low
	   		  DrvADC_Enable();
	   		  DrvADC_CombFilter(ENABLE);  //Disable CombFilter
	   		  DrvADC_ClearIntFlag();
	   		  DrvADC_EnableInt();
	   		  while(!ADfinish);

	   	    }

	   		if(i==8 && DAC_Calibration_ADCData>= ADCData)  //if Vin>Vdac, Vdac Output code + 1
	   	    {
	   		  DrvADC_Disable();
	   		  DrvADC_CombFilter(DISABLE);  //Disable CombFilter
	   		  Vdac_Buffer=1;
	   		  Vdac_Output=dac_04;
	   		  Vdac_Output=Vdac_Output+Vdac_Buffer;
	   		  dac_04=0xff00+Vdac_Output;  //To do Vdac output voltage
	   		  DrvFlash_Burn_Word(0xf000,2000,Vdac_Output);  //DAC calibration Burn_Word at 0x9f000
	   		  DAC_Calibration_Flag=1; //If DAC_Calibration_Flag=1, leave while((pio1_3<=PT1_7_low)&&(DAC_Calibration_Flag==0))
              // check DAC calibration time
              //DrvGPIO_SetPortBits(E_PT2,0x08);  //PT2.3=High
              //Delay(1000); //wait ? second
              //DrvGPIO_ClrPortBits(E_PT2,0x08);  //PT2.3=Low
	   		  DrvADC_Enable();
	   		  DrvADC_CombFilter(ENABLE);  //Disable CombFilter
	   		  DrvADC_ClearIntFlag();
	   		  DrvADC_EnableInt();
	   		  while(!ADfinish);

	   	    }

	   		ADfinish=0;
	   		MCUSTATUSbits.b_ADCdone=0;

	   	}
#endif

	  }

  }


#if defined(ADC_ohm_show_19bit)
//Ohm table established in  DAC=330mV
//ADC setting : DrvADC_FullRefRange(1), OSR(0)
  K_ohm[0]=3.00;  //3k ohm
  //K_ohm[1]=30.00;  //30k ohm
  //K_ohm[2]=75.00;  //75k ohm
  //K_ohm[3]=300.00;  //300k ohm
  K_ohm_ADCount[0]=151790;  //3k ohm, ADC Count
  //K_ohm_ADCount[1]=15xxx;  //30k
  //K_ohm_ADCount[2]=60xx;  //75k
  //K_ohm_ADCount[3]=1xx;  //300k
#endif

  //Start to do ADC Measure RS_Strip

  DrvADC_Disable();
  DrvADC_CombFilter(DISABLE);  //Disable CombFilter

  //DAC Output Setting
  if(DAC_Calibration_Flag==0)  //Use default DAC output value
  {
    DrvDAC_Open(E_DAC_PAIO4,E_DAC_NAIO7,DAC_Output);
    DrvDAC_SetoutputIO(ENABLE);       //DAC output with PT3.1
    DrvDAC_EnableOutput();            //DAC output enable
    DrvDAC_Enable();                  //DAC IP enable
  }

  //DAC Output Setting
  if(DAC_Calibration_Flag==1)  //Use Calibrated DAC output value
  {
	DAC_Output=ReadWord(0xf000);  //Read out Flash address 0x9f000
    DrvDAC_Open(E_DAC_PAIO4,E_DAC_NAIO7,DAC_Output);
    DrvDAC_SetoutputIO(ENABLE);       //DAC output with PT3.1
    DrvDAC_EnableOutput();            //DAC output enable
    DrvDAC_Enable();                  //DAC IP enable
  }

  // R2ROPA Setting
  DrvOP_PInput(E_OPP_DAO);
  DrvOP_NInput(E_OPN_AIO5);
  DrvOP_Open();
  DrvOP_OPOoutEnable(); //OPA OPEO=1


  InitalADC();  //initial ADC : P=AIO0, N=AIO5
  DisplayInit();
  ClearLCDframe();

  SYS_EnableGIE(7,0x1FF);  //Enable GIE(Global Interrupt)
  MCUSTATUSbits._byte = 0;
  PT1INTSTATUSbits._byte = 0;


  while(1)
  {

#if defined(ADC_Count_show_19bit)
	 //No ADC chopper
	 /*
	 if(MCUSTATUSbits.b_ADCdone)
	 {
	    while(!ADfinish);
	    LCD_DATA_DISPLAY(ADCData);	//to show ADC_Count_show_19bit
	    ADfinish=0;
	    MCUSTATUSbits.b_ADCdone=0;
	 }
	 */

	  //ADC chopper
	  for(i=0;i<=1;i++)
	  {
	 	 switch (i)
	 	 {
	 	 	case 0:  //已經抓到第一筆
	 	 	while(!ADfinish);  //if ADfinish=0, stop at here, until enter ADC INT
	 	 	ADfinish=0;
	 	 	ADC_Array[0]=ADCData;
	 	 	DrvADC_Disable();
	 	 	DrvADC_CombFilter(DISABLE);  //Disable CombFilter
	 	 	DrvADC_SetADCInputChannel(ADC_Input_AIO5,ADC_Input_AIO0); //Set the ADC positive/negative input voltage source.
	 	 	DrvADC_Enable();
	 	 	DrvADC_CombFilter(ENABLE);  //Disable CombFilter
	 	 	DrvADC_ClearIntFlag();
	 	 	DrvADC_EnableInt();
	 	 	while(!ADfinish);  //if ADfinish=0, stop at here
	 	 	break;

	 	 	case 1:  //已經抓到第二筆
	 	 	ADfinish=0;
	 	 	ADC_Array[1]=ADCData;
	 	 	DrvADC_Disable();
	 	 	DrvADC_CombFilter(DISABLE);  //Disable CombFilter
	 	 	ADCData_Chopper=(ADC_Array[0]-ADC_Array[1])>>1;	//divider by 2
	 	 	LCD_DATA_DISPLAY(ADCData_Chopper);
	 	 	DrvADC_SetADCInputChannel(ADC_Input_AIO0,ADC_Input_AIO5); //Set the ADC positive/negative input voltage source.
	 	 	DrvADC_Enable();
	 	 	DrvADC_CombFilter(ENABLE);  //Disable CombFilter
	 	 	DrvADC_ClearIntFlag();
	 	 	DrvADC_EnableInt();
	 	 	while(!ADfinish);	 //if ADfinish=0, stop at here
	 	 	break;
	 	  }
	   }

#endif


#if defined(ADC_ohm_show_19bit)

	//No ADC chopper mode
	/*
    if(MCUSTATUSbits.b_ADCdone)
    {
          while(!ADfinish);
          ADCData_temp=ADCData;
          //Case1 : Abnormal case, show 0 ohm
          if((ADCData_temp==ADC_over_19bit)) //that means ADC Count over
          {
            ADCData_ohm_show=0;
            LCD_Ohm_DISPLAY(ADCData_ohm_show); // for example : ADCData_ohm_show(3k)=3000
          }
          //Case2 : Abnormal case, show 0 ohm
          if(ADCData_temp<=0)  //it is abnormal case
          {
        	ADCData_temp=0;
        	ADCData_ohm_show=ADCData_temp;
            LCD_Ohm_DISPLAY(ADCData_ohm_show); // for example : ADCData_ohm_show(3k)=3000
          }
          //Case3 : Normal case.
          if( (ADCData_temp>0) && (ADCData_temp!= ADC_over_19bit)) //It is normal case
          {
            ADCData_ohm_show=(K_ohm_ADCount[0]/ADCData_temp)*K_ohm[0];
            ADCData_ohm_show=ADCData_ohm_show*1000;

            if(ADCData_ohm_show>=500000)
            {
        	  ADCData_ohm_show=0;
        	  LCD_Ohm_DISPLAY(ADCData_ohm_show); // for example : ADCData_ohm_show(3k)=3000

            }
              LCD_Ohm_DISPLAY(ADCData_ohm_show); // for example : ADCData_ohm_show(3k)=3000

          }
          ADfinish=0;
          MCUSTATUSbits.b_ADCdone=0;
      }
      */

	  //ADC chopper mode
	  for(i=0;i<=1;i++)
	  {
	  	 switch (i)
	  	 {
	  	 	case 0:  //已經抓到第一筆
	  	 	while(!ADfinish);  //if ADfinish=0, stop at here, until enter ADC INT
	  	 	ADfinish=0;
	  	 	ADC_Array[0]=ADCData;
	  	 	DrvADC_Disable();
	  	 	DrvADC_CombFilter(DISABLE);  //Disable CombFilter
	  	 	DrvADC_SetADCInputChannel(ADC_Input_AIO5,ADC_Input_AIO0); //Set the ADC positive/negative input voltage source.
	  	 	DrvADC_Enable();
	  	 	DrvADC_CombFilter(ENABLE);  //Disable CombFilter
	  	 	DrvADC_ClearIntFlag();
	  	 	DrvADC_EnableInt();
	  	 	while(!ADfinish);  //if ADfinish=0, stop at here
	  	 	break;

	  	 	case 1:  //已經抓到第二筆
	  	 	ADfinish=0;
	  	 	ADC_Array[1]=ADCData;
	  	 	DrvADC_Disable();
	  	 	DrvADC_CombFilter(DISABLE);  //Disable CombFilter
	  	 	ADCData_Chopper=(ADC_Array[0]-ADC_Array[1])>>1;	//divider by 2
	  	 	ADCData_temp=ADCData_Chopper;

	  	      //Case1 : Abnormal case, show 0 ohm
	  	      if((ADCData_temp==ADC_over_19bit)) //that means ADC Count over
	  	      {
	  	        ADCData_ohm_show=0;
	  	        LCD_Ohm_DISPLAY(ADCData_ohm_show); // for example : ADCData_ohm_show(3k)=3000
	  	      }
	  	      //Case2 : Abnormal case, show 0 ohm
	  	      if(ADCData_temp<=0)  //it is abnormal case
	  	      {
	  	        ADCData_temp=0;
	  	        ADCData_ohm_show=ADCData_temp;
	  	        LCD_Ohm_DISPLAY(ADCData_ohm_show); // for example : ADCData_ohm_show(3k)=3000
	  	       }
	  	       //Case3 : Normal case.
	  	       if( (ADCData_temp>0) && (ADCData_temp!= ADC_over_19bit)) //It is normal case
	  	       {
	  	         ADCData_ohm_show=(K_ohm_ADCount[0]/ADCData_temp)*K_ohm[0];
	  	         ADCData_ohm_show=ADCData_ohm_show*1000;
	  	         if(ADCData_ohm_show>=500000)
	  	         {
	  	            ADCData_ohm_show=0;
	  	            LCD_Ohm_DISPLAY(ADCData_ohm_show); // for example : ADCData_ohm_show(3k)=3000
	  	         }
	  	            LCD_Ohm_DISPLAY(ADCData_ohm_show); // for example : ADCData_ohm_show(3k)=3000
	  	       }
	  	    DrvADC_SetADCInputChannel(ADC_Input_AIO0,ADC_Input_AIO5); //Set the ADC positive/negative input voltage source.
	  	 	DrvADC_Enable();
	  	 	DrvADC_CombFilter(ENABLE);  //Disable CombFilter
	  	 	DrvADC_ClearIntFlag();
	  	 	DrvADC_EnableInt();
	  	 	while(!ADfinish);	 //if ADfinish=0, stop at here
	  	 	break;
	  	 }
	  }

#endif

  }
  return 0;
}

/*--------------------------------------------------------------------*/
/* ADC Interrupt Subroutines                                          */
/*--------------------------------------------------------------------*/
void HW2_ISR(void)
{
  if(DrvADC_ReadIntFlag()==1)
  {
    DrvADC_ClearIntFlag();
    ADCData=DrvADC_GetConversionData();
    ADCData=ADCData>>13; //To get high 19 bits ADC output code only
    MCUSTATUSbits.b_ADCdone=1;
    ADfinish=1;
  }

}

/*---------------------------------------------------------------------------*/
/* PT1 Interrupt Service Routines                                            */
/*---------------------------------------------------------------------------*/
void HW4_ISR(void)
{
  uint32_t PORT_IntFlag;
  PORT_IntFlag=DrvGPIO_GetIntFlag(E_PT1);
  if((PORT_IntFlag&0x02)==0x02)  // Enter PT1 INT to check PT1.1 INT Flag
  {
	PT1INTSTATUSbits.b_PTINT0done=1;
  }
    DrvGPIO_ClearIntFlag(E_PT1,0x02);  //clear PT1.1 interrupt flag
}
/*--------------------------------------------------------------------*/
/* ADC Initialization Subroutines                                     */
/*--------------------------------------------------------------------*/
void InitalADC(void)
{
  DrvADC_ClkEnable(0,1);       //Setting ADC CLOCK ADCK=HS_CK/6 & Rising edge is high
  //Set VDDA voltage
  DrvPMU_VDDA_LDO_Ctrl(E_LDO);
  DrvPMU_VDDA_Voltage(E_VDDA2_4);
  DrvPMU_BandgapEnable();
  //Enable REFO
  DrvPMU_REFO_Enable();

  Delay(0x1000);
  DrvPMU_AnalogGround(ENABLE);      //ADC analog ground source selection.
	                                //1 : Enable buffer and use internal source(need to work with ADC)

  //Set ADC input pin
  DrvADC_SetADCInputChannel(ADC_Input_AIO0,ADC_Input_AIO5); //Set the ADC positive/negative input voltage source.
  DrvADC_InputSwitch(OPEN);                //ADC signal input (positive and negative) short(VISHR) control.
  DrvADC_RefInputShort(OPEN);              //Set the ADC reference input (positive and negative) short(VRSHR) control.

  DrvADC_Gain(ADC_PGA_Disable,ADC_PGA_Disable); //Input signal gain for modulator.
  DrvADC_DCoffset(0);                      //DC offset input voltage selection (VREF=REFP-REFN)
  DrvADC_RefVoltage(0,3);  				   //Set the ADC reference voltage. VDDA-REFO
  DrvADC_FullRefRange(1);                  //Set the ADC full reference range select.
                                           //0: Full reference range input
                                           //1: 1/2 reference range input
  //DrvADC_OSR(0);              	       //0 : OSR=32768, output rate=10Hz
  DrvADC_OSR(1);              	           //1 : OSR=16384, output rate=20Hz

  //Set ADC interrupt
  DrvADC_ClearIntFlag();
  DrvADC_EnableInt();
  DrvADC_Enable();

  DrvADC_CombFilter(ENABLE);   //Enable comb filter
  DrvADC_CombFilter(DISABLE);   //DISABLE comb filter
  DrvADC_CombFilter(ENABLE);   //Enable comb filter
}
/*--------------------------------------------------------------------*/
/* ADC Initialization Subroutines                                     */
/*--------------------------------------------------------------------*/
void InitalADC_DAC_Calibration(void)
{
  DrvADC_ClkEnable(0,1);       //Setting ADC CLOCK ADCK=HS_CK/6 & Rising edge is high
  //Set VDDA voltage
  DrvPMU_VDDA_LDO_Ctrl(E_LDO);
  DrvPMU_VDDA_Voltage(E_VDDA2_4);
  DrvPMU_BandgapEnable();
  //Enable REFO
  DrvPMU_REFO_Enable();

  Delay(0x1000);
  DrvPMU_AnalogGround(ENABLE);      //ADC analog ground source selection.
	                                //1 : Enable buffer and use internal source(need to work with ADC)

  //Set ADC input pin
  DrvADC_SetADCInputChannel(ADC_Input_AIO5,REFO_I); //Set the ADC positive/negative input voltage source.

  DrvADC_InputSwitch(OPEN);                //ADC signal input (positive and negative) short(VISHR) control.
  DrvADC_RefInputShort(OPEN);              //Set the ADC reference input (positive and negative) short(VRSHR) control.

  DrvADC_Gain(ADC_PGA_Disable,ADC_PGA_Disable); //Input signal gain for modulator.
  DrvADC_DCoffset(0);                      //DC offset input voltage selection (VREF=REFP-REFN)
  DrvADC_RefVoltage(0,3); 				   //Set the ADC reference voltage. VDDA-REFO
  DrvADC_FullRefRange(1);                  //Set the ADC full reference range select.
                                           //0: Full reference range input
                                           //1: 1/2 reference range input
  //DrvADC_OSR(0);             			   //0 : OSR=32768, output rate=10Hz
  //DrvADC_OSR(1);              	       //1 : OSR=16384, output rate=20Hz
  DrvADC_OSR(5);              	           //5 : OSR=1024, output rate=326Hz

  //Set ADC interrupt
  DrvADC_ClearIntFlag();
  DrvADC_EnableInt();
  DrvADC_Enable();

  DrvADC_CombFilter(ENABLE);   //Enable comb filter
  DrvADC_CombFilter(DISABLE);   //DISABLE comb filter
  DrvADC_CombFilter(ENABLE);   //Enable comb filter
}

/*---------------------------------------------------------------------------*/
/* Software Delay Subroutines                                                */
/*---------------------------------------------------------------------------*/
void Delay(unsigned int num)
{
  for(;num>0;num--)
  asm("NOP");
}

/*---------------------------------------------------------------------------*/
/* Exception Service Routines                                                */
/*---------------------------------------------------------------------------*/
void tlb_exception_handler()
{
  asm("nop"); //procedure define by customer.
  asm("nop");
}

/*---------------------------------------------------------------------------*/
/* End Of File                                                               */
/*---------------------------------------------------------------------------*/
